@@ -17,6 +17,7 @@
 __author__ = 'Joseph DiLallo'
 
 import os
+import StringIO
 import sys
 import urllib
 import urllib2
@@ -112,6 +113,45 @@ _SERVICE_MAP = {
         'TargetingIdeaService': 'o',
         'TrafficEstimatorService': 'o',
     },
+    'v201409': {
+        'AdGroupAdService': 'cm',
+        'AdGroupBidModifierService': 'cm',
+        'AdGroupCriterionService': 'cm',
+        'AdGroupFeedService': 'cm',
+        'AdGroupService': 'cm',
+        'AdParamService': 'cm',
+        'AdwordsUserListService': 'rm',
+        'BiddingStrategyService': 'cm',
+        'BudgetOrderService': 'billing',
+        'BudgetService': 'cm',
+        'CampaignAdExtensionService': 'cm',
+        'CampaignCriterionService': 'cm',
+        'CampaignFeedService': 'cm',
+        'CampaignService': 'cm',
+        'CampaignSharedSetService': 'cm',
+        'ConstantDataService': 'cm',
+        'ConversionTrackerService': 'cm',
+        'CustomerFeedService': 'cm',
+        'CustomerService': 'mcm',
+        'CustomerSyncService': 'ch',
+        'DataService': 'cm',
+        'ExperimentService': 'cm',
+        'FeedItemService': 'cm',
+        'FeedMappingService': 'cm',
+        'FeedService': 'cm',
+        'GeoLocationService': 'cm',
+        'LabelService': 'cm',
+        'LocationCriterionService': 'cm',
+        'ManagedCustomerService': 'mcm',
+        'MediaService': 'cm',
+        'MutateJobService': 'cm',
+        'OfflineConversionFeedService': 'cm',
+        'ReportDefinitionService': 'cm',
+        'SharedCriterionService': 'cm',
+        'SharedSetService': 'cm',
+        'TargetingIdeaService': 'o',
+        'TrafficEstimatorService': 'o',
+    },
 }
 
 # The endpoint used by default when making AdWords API requests.
@@ -282,6 +322,14 @@ class AdWordsClient(object):
     """
     return ReportDownloader(self, version, server)
 
+  def SetClientCustomerId(self, client_customer_id):
+    """Change the client customer id used by the AdWordsClient instance.
+
+    Args:
+      client_customer_id: str New Client Customer Id to use.
+    """
+    self.client_customer_id = client_customer_id
+
 
 class _AdWordsHeaderHandler(googleads.common.HeaderHandler):
   """Handler which generates the headers for AdWords requests."""
@@ -321,13 +369,21 @@ class _AdWordsHeaderHandler(googleads.common.HeaderHandler):
         soapheaders=header,
         headers=self._adwords_client.oauth2_client.CreateHttpHeader())
 
-  def GetReportDownloadHeaders(self, return_money_in_micros=None):
+  def GetReportDownloadHeaders(self, return_money_in_micros=None,
+                               skip_report_header=None,
+                               skip_report_summary=None):
     """Returns a dictionary of headers for a report download request.
 
     Args:
       return_money_in_micros: A boolean indicating whether money should be
           represented as micros in reports. If None is supplied the AdWords
           server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
 
     Returns:
       A dictionary containing the headers configured for downloading a report.
@@ -344,6 +400,13 @@ class _AdWordsHeaderHandler(googleads.common.HeaderHandler):
         'User-Agent': ''.join([self._adwords_client.user_agent, self._LIB_SIG,
                                ',gzip'])
     })
+
+    if skip_report_header:
+      headers.update({'skipReportHeader': str(skip_report_header)})
+
+    if skip_report_summary:
+      headers.update({'skipReportSummary': str(skip_report_summary)})
+
     if return_money_in_micros is not None:
       if self._version == _FINAL_RETURN_MONEY_IN_MICROS_VERSION:
         headers.update({'returnMoneyInMicros': str(return_money_in_micros)})
@@ -408,9 +471,9 @@ class ReportDownloader(object):
       raise googleads.errors.GoogleAdsValueError('Need to specify a binary'
                                                  ' output for GZIPPED formats.')
 
-
   def DownloadReport(self, report_definition, output=sys.stdout,
-                     return_money_in_micros=None):
+                     return_money_in_micros=None, skip_report_header=None,
+                     skip_report_summary=None):
     """Downloads an AdWords report using a report definition.
 
     The report contents will be written to the given output.
@@ -426,6 +489,12 @@ class ReportDownloader(object):
       return_money_in_micros: A boolean indicating whether money should be
           represented as micros in reports. If None is supplied the AdWords
           server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
 
     Raises:
       AdWordsReportBadRequestError: if the report download fails due to
@@ -437,10 +506,176 @@ class ReportDownloader(object):
     """
     self._DownloadReportCheckFormat(report_definition['downloadFormat'], output)
     self._DownloadReport(self._SerializeReportDefinition(report_definition),
-                         output, return_money_in_micros)
+                         output, return_money_in_micros, skip_report_header,
+                         skip_report_summary)
+
+  def DownloadReportAsStream(self, report_definition,
+                             return_money_in_micros=None,
+                             skip_report_header=None, skip_report_summary=None):
+    """Downloads an AdWords report using a report definition.
+
+    This will return a stream, allowing you to retrieve the report contents.
+
+    Args:
+      report_definition: A dictionary or instance of the ReportDefinition class
+          generated from the schema. This defines the contents of the report
+          that will be downloaded.
+      [optional]
+      return_money_in_micros: A boolean indicating whether money should be
+          represented as micros in reports. If None is supplied the AdWords
+          server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
+
+    Returns:
+      A stream to be used in retrieving the report contents.
+
+    Raises:
+      AdWordsReportBadRequestError: if the report download fails due to
+          improper input.
+      GoogleAdsValueError: if the user-specified report format is incompatible
+          with the output.
+      AdWordsReportError: if the request fails for any other reason; e.g. a
+          network error.
+    """
+    return self._DownloadReportAsStream(
+        self._SerializeReportDefinition(report_definition),
+        return_money_in_micros, skip_report_header, skip_report_summary)
+
+  def DownloadReportAsStreamWithAwql(self, query, file_format,
+                                     return_money_in_micros=None,
+                                     skip_report_header=None,
+                                     skip_report_summary=None):
+    """Downloads an AdWords report using an AWQL query.
+
+    The report contents will be returned as a stream.
+
+    Args:
+      query: A string containing the query which specifies the data you want
+          your report to include.
+      file_format: A string representing the output format for your report.
+          Acceptable values can be found in our API documentation:
+          https://developers.google.com/adwords/api/docs/guides/reporting
+      [optional]
+      return_money_in_micros: A boolean indicating whether money should be
+          represented as micros in reports. If None is supplied the AdWords
+          server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
+
+    Returns:
+      A stream to be used in retrieving the report contents.
+
+    Raises:
+      AdWordsReportBadRequestError: if the report download fails due to
+          improper input.
+      GoogleAdsValueError: if the user-specified report format is incompatible
+          with the output.
+      AdWordsReportError: if the request fails for any other reason; e.g. a
+          network error.
+    """
+    return self._DownloadReportAsStream(self._SerializeAwql(query, file_format),
+                                        return_money_in_micros,
+                                        skip_report_header, skip_report_summary)
+
+  def DownloadReportAsString(self, report_definition,
+                             return_money_in_micros=None,
+                             skip_report_header=None, skip_report_summary=None):
+    """Downloads an AdWords report using a report definition.
+
+    The report contents will be returned as a string.
+
+    Args:
+      report_definition: A dictionary or instance of the ReportDefinition class
+          generated from the schema. This defines the contents of the report
+          that will be downloaded.
+      [optional]
+      return_money_in_micros: A boolean indicating whether money should be
+          represented as micros in reports. If None is supplied the AdWords
+          server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
+
+    Returns:
+      A string containing the report contents.
+
+    Raises:
+      AdWordsReportBadRequestError: if the report download fails due to
+          improper input.
+      GoogleAdsValueError: if the user-specified report format is incompatible
+          with the output.
+      AdWordsReportError: if the request fails for any other reason; e.g. a
+          network error.
+    """
+    try:
+      response = StringIO.StringIO()
+      self._DownloadReport(self._SerializeReportDefinition(report_definition),
+                           response, return_money_in_micros, skip_report_header,
+                           skip_report_summary)
+      return response.getvalue()
+    finally:
+      response.close()
+
+  def DownloadReportAsStringWithAwql(self, query, file_format,
+                                     return_money_in_micros=None,
+                                     skip_report_header=None,
+                                     skip_report_summary=None):
+    """Downloads an AdWords report using an AWQL query.
+
+    The report contents will be returned as a string.
+
+    Args:
+      query: A string containing the query which specifies the data you want
+          your report to include.
+      file_format: A string representing the output format for your report.
+          Acceptable values can be found in our API documentation:
+          https://developers.google.com/adwords/api/docs/guides/reporting
+      [optional]
+      return_money_in_micros: A boolean indicating whether money should be
+          represented as micros in reports. If None is supplied the AdWords
+          server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
+
+    Returns:
+      A string containing the report contents.
+
+    Raises:
+      AdWordsReportBadRequestError: if the report download fails due to
+          improper input.
+      GoogleAdsValueError: if the user-specified report format is incompatible
+          with the output.
+      AdWordsReportError: if the request fails for any other reason; e.g. a
+          network error.
+    """
+    try:
+      response = StringIO.StringIO()
+      self._DownloadReport(self._SerializeAwql(query, file_format), response,
+                           return_money_in_micros, skip_report_header,
+                           skip_report_summary)
+      return response.getvalue()
+    finally:
+      response.close()
 
   def DownloadReportWithAwql(self, query, file_format, output=sys.stdout,
-                             return_money_in_micros=None):
+                             return_money_in_micros=None,
+                             skip_report_header=None, skip_report_summary=None):
     """Downloads an AdWords report using an AWQL query.
 
     The report contents will be written to the given output.
@@ -458,6 +693,12 @@ class ReportDownloader(object):
       return_money_in_micros: A boolean indicating whether money should be
           represented as micros in reports. If None is supplied the AdWords
           server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
 
     Raises:
       AdWordsReportBadRequestError: if the report download fails due to
@@ -469,9 +710,11 @@ class ReportDownloader(object):
     """
     self._DownloadReportCheckFormat(file_format, output)
     self._DownloadReport(self._SerializeAwql(query, file_format), output,
-                         return_money_in_micros)
+                         return_money_in_micros, skip_report_header,
+                         skip_report_summary)
 
-  def _DownloadReport(self, post_body, output, return_money_in_micros):
+  def _DownloadReport(self, post_body, output, return_money_in_micros,
+                      skip_report_header, skip_report_summary):
     """Downloads an AdWords report, writing the contents to the given file.
 
     Args:
@@ -482,6 +725,49 @@ class ReportDownloader(object):
       return_money_in_micros: A boolean indicating whether money should be
           represented as micros in reports. If None is supplied the AdWords
           server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
+
+    Raises:
+      AdWordsReportBadRequestError: if the report download fails due to
+        improper input. In the event of certain other failures, a
+        urllib2.URLError (Python 2) or urllib.error.URLError (Python 3) will be
+        raised.
+      AdWordsReportError: if the request fails for any other reason; e.g. a
+          network error.
+    """
+    response = self._DownloadReportAsStream(post_body, return_money_in_micros,
+                                            skip_report_header,
+                                            skip_report_summary)
+    while True:
+      chunk = response.read(_CHUNK_SIZE)
+      if not chunk: break
+      output.write(chunk.decode() if sys.version_info[0] == 3
+                   and getattr(output, 'mode', 'w') == 'w' else chunk)
+
+  def _DownloadReportAsStream(self, post_body, return_money_in_micros,
+                              skip_report_header, skip_report_summary):
+    """Downloads an AdWords report, returning a stream.
+
+    Args:
+      post_body: The contents of the POST request's body as a URL encoded
+          string.
+      return_money_in_micros: A boolean indicating whether money should be
+          represented as micros in reports. If None is supplied the AdWords
+          server will use its default value, which is currently True.
+      skip_report_header: A boolean indicating whether to include a header row
+          containing the report name and date range. If false or not specified,
+          report output will include the header row.
+      skip_report_summary: A boolean indicating whether to include a summary row
+          containing the report totals. If false or not specified, report output
+          will include the summary row.
+
+    Returns:
+      A stream to be used in retrieving the report contents.
 
     Raises:
       AdWordsReportBadRequestError: if the report download fails due to
@@ -495,19 +781,15 @@ class ReportDownloader(object):
       post_body = bytes(post_body, 'utf8')
     request = urllib2.Request(
         self._end_point, post_body,
-        self._header_handler.GetReportDownloadHeaders(return_money_in_micros))
+        self._header_handler.GetReportDownloadHeaders(return_money_in_micros,
+                                                      skip_report_header,
+                                                      skip_report_summary))
     if self._adwords_client.https_proxy:
       request.set_proxy(self._adwords_client.https_proxy, 'https')
     try:
-      response = urllib2.urlopen(request)
+      return urllib2.urlopen(request)
     except urllib2.HTTPError, e:
       raise self._ExtractError(e)
-    else:
-      while True:
-        chunk = response.read(_CHUNK_SIZE)
-        if not chunk: break
-        output.write(chunk.decode() if sys.version_info[0] == 3
-                     and getattr(output, 'mode', 'w') == 'w' else chunk)
 
   def _SerializeAwql(self, query, file_format):
     """Serializes an AWQL query and file format for transport.
